@@ -7,7 +7,7 @@
  *
  * The GnuTLS is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 3 of
+ * as published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful, but
@@ -149,7 +149,7 @@ _gnutls_buffer_append_data (gnutls_buffer_st * dest, const void *data,
       size_t new_len =
         MAX (data_size, MIN_CHUNK) + MAX (dest->max_length, MIN_CHUNK);
 
-      dest->allocd = gnutls_realloc (dest->allocd, new_len);
+      dest->allocd = gnutls_realloc_fast (dest->allocd, new_len);
       if (dest->allocd == NULL)
         {
           gnutls_assert ();
@@ -190,7 +190,7 @@ _gnutls_buffer_resize (gnutls_buffer_st * dest, size_t new_size)
       size_t alloc_len =
         MAX (new_size, MIN_CHUNK) + MAX (dest->max_length, MIN_CHUNK);
 
-      dest->allocd = gnutls_realloc (dest->allocd, alloc_len);
+      dest->allocd = gnutls_realloc_fast (dest->allocd, alloc_len);
       if (dest->allocd == NULL)
         {
           gnutls_assert ();
@@ -301,6 +301,11 @@ _gnutls_buffer_pop_data (gnutls_buffer_st * str, void *data,
   gnutls_datum_t tdata;
 
   _gnutls_buffer_pop_datum (str, &tdata, *req_size);
+  if (tdata.data == NULL)
+    {
+      *req_size = 0;
+      return;
+    }
 
   *req_size = tdata.size;
   memcpy (data, tdata.data, tdata.size);
@@ -534,6 +539,67 @@ _gnutls_hex2bin (const char * hex_data, size_t hex_size, uint8_t * bin_data,
   return 0;
 }
 
+/**
+ * gnutls_hex_decode:
+ * @hex_data: contain the encoded data
+ * @result: the place where decoded data will be copied
+ * @result_size: holds the size of the result
+ *
+ * This function will decode the given encoded data, using the hex
+ * encoding used by PSK password files.
+ *
+ * Note that hex_data should be null terminated.
+ *
+ * Returns: %GNUTLS_E_SHORT_MEMORY_BUFFER if the buffer given is not
+ *   long enough, or 0 on success.
+ **/
+int
+gnutls_hex_decode (const gnutls_datum_t * hex_data, void *result,
+                   size_t * result_size)
+{
+  int ret;
+
+  ret =
+    _gnutls_hex2bin ((char*)hex_data->data, hex_data->size, (uint8_t *) result,
+                     result_size);
+  if (ret < 0)
+    return ret;
+
+  return 0;
+}
+
+/**
+ * gnutls_hex_encode:
+ * @data: contain the raw data
+ * @result: the place where hex data will be copied
+ * @result_size: holds the size of the result
+ *
+ * This function will convert the given data to printable data, using
+ * the hex encoding, as used in the PSK password files.
+ *
+ * Note that the size of the result includes the null terminator.
+ *
+ * Returns: %GNUTLS_E_SHORT_MEMORY_BUFFER if the buffer given is not
+ * long enough, or 0 on success.
+ **/
+int
+gnutls_hex_encode (const gnutls_datum_t * data, char *result,
+                   size_t * result_size)
+{
+  size_t res = data->size + data->size + 1;
+
+  if (*result_size < res)
+    {
+      gnutls_assert ();
+      return GNUTLS_E_SHORT_MEMORY_BUFFER;
+    }
+
+  _gnutls_bin2hex (data->data, data->size, result, *result_size, NULL);
+  *result_size = res;
+
+  return 0;
+}
+
 
 /* compare hostname against certificate, taking account of wildcards
  * return 1 on success or 0 on error
